package org.jvisu.base;

import java.awt.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values
 * <p>
 * $Id: JVDigits.java 311 2006-06-23 10:22:28Z mol $
 * </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  The purpose of JVDigits is to display a rectangular shape, which contains
 *  the numerical representation of a variable. The representation can be 
 *  customized by various methods.
 *  The components has a builtin min/max compare function, which can be custumized by 
 *  absolute values or variable values, which are retrieved from the communication server
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVDigits extends JVDouble implements PropertyChangeListener {
 
    /** Color for text display */    
    protected Color             textColor = Color.black;
    /** background color for alarm */    
    protected Color             alarmColor = Color.red;
    /** unit to append to the value */    
    private String              unit;
    /** Textfield to edit the variable value, if edit mode is active */    
    private JTextField          edit;
    
    /** Create a new instance of VisuDigits
     */
    public JVDigits() {
        // Add a textfield, as we could be a editable component
        setLayout(new java.awt.GridLayout());
        edit = new JTextField("0");
        add (edit);
        // The editfield is invisble by default
        edit.setVisible(false);
        edit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                varEdited();
            }
        });
        this.setDesignsize(new Dimension (100,30));
    }

    /** Variable has been changed in editor, do range checking. 
     *  TODO: Popup display for wrong input values
     */
    private void varEdited (){
        double val = JVParams.StringToDouble(edit.getText(), actval);
        if (val < minval)
                val = minval;
        if (val > maxval)
                val = maxval;
        // VarChanged will only be called if the value _really_ changed.
        // So set the value to make sure the format is correct.
        edit.setText(numFormat.format(val));
        setValue (val);
    }

    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        if (key.equals("editEnabled") || key.equals("readonly")) {
            if (!isEditEnabled() || isReadonly()) {
                setToolTipText (null);
            } else {
                String tip = "Enter new value";
                if (minval != Double.MIN_VALUE)
                    tip += " Min: " + minval;
                if (maxval != Double.MAX_VALUE)
                    tip += " Max: " + maxval;
                setToolTipText(tip);
            }
            edit.setVisible(isEditEnabled() && !isReadonly());
            edit.setFont (this.getFont());    
            repaint();
        }
        if (key.equals("font"))
            edit.setFont (this.getFont());    
    }
    
    /** Get text color
     *  @return text color
     */
    public Color getTextColor (){
        return textColor;
    }

    /** Set text color
     *  @param col text color
     */
    public void setTextColor (Color col){
        textColor = col;
    }

    /** Get color for alarm
     *  @return alarm color
     */
    public Color getAlarmColor (){
        return alarmColor;
    }

    /** Set color for alarm
     *  @param col alarm color
     */
    public void setAlarmColor (Color col){
        alarmColor = col;
    }
    
    /** Get unit string
    *  @return unit String, which is appended to the display value
    */
    public String getUnit () {
        return this.unit;
    }

   /** Set unit string
    *  @param unit unit String, which is appended to the display value
    */
    public void setUnit (String unit) {
        this.unit = unit;
    }

    /** Informes the owner, that the variable value has been changed
     * @param var Variable to display
     */
    public void VarChanged(JVVar var) {
        super.VarChanged(var);
        edit.setText(numFormat.format(actval));
    }
    
    /** VisuDigits specific paint function.
     *  Check, if a min/max check is given<br>
     *  If yes, check, if value is inside the tresholds.<br>
     *  If inside use backcolor, else use alarmcolor
     * @param g Graphics context
     */
    public void paintComponent (Graphics g) {
        
        if (isEditEnabled() && !isReadonly()) {
            super.paintComponent(g);
            return;
        }
        Dimension size = getSize();
        
        // convert the value to the requested number format
        String txt = numFormat.format (actval);
        // append unit-string, if available
        if (unit != null)
            txt += " " + unit;

        Color col = this.isOpaque() ? this.getBackground() : null;

        // check, if we have to draw the background with alarm color
        if (actval < minval)
            col = alarmColor;
        if (actval > maxval)
                 col = alarmColor;
        if (col != null && this.isOpaque()) {
            g.setColor (col);
            g.fillRect (0, 0, size.width, size.height);
        }

        g.setColor (textColor);
        g.setFont(this.getFont());

        // retrieve textsize
        int fheight = g.getFontMetrics ().getHeight ();
        int fwidth = g.getFontMetrics ().stringWidth (txt);
        
        int x = 0; 
        // set x startpoint depending on alignement
        switch (getAligncontent()) {
            // align left
            case JVAlignment.JVISU_ALIGNLEFT: x = g.getFontMetrics ().charWidth(' '); break;
            // align center
            case JVAlignment.JVISU_ALIGNCENTER: x = (size.width - fwidth) / 2; break;
            // align right
            default: x = size.width - fwidth - g.getFontMetrics ().charWidth(' ');
        }
        g.drawString (txt, x, (size.height/2 + fheight/3));
    }
}
