package org.jvisu.comm.var;

import java.lang.*;
import java.util.*;
import java.nio.*;

/** Abstract base class for all JVisu variables.
 *
 *  <p>package:  org.jvisu.comm.var</p>
 *  <p>file:    JVVar.java</p>
 *  <p>$Id: JVVar.java 302 2005-10-16 10:43:06Z ero $ </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  Abstract base class for all JVisu variable types. JVVar implements
 *  the type independent properties and methods and declares the
 *  abstract methods, which have to be implemented by all implementations
 *  of this class.
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public abstract class JVVar {
    
    // variable type constants
    
    /** JVisu variable of type double */
    public  static final int T_DOUBLE = 1;
    /** JVisu variable of type string */
    public  static final int T_STRING = 2;
    /** JVisu variable of type bool */
    public  static final int T_BOOL = 3;
    /** JVisu variable of type byte */
    public  static final int T_BYTE = 4;
    /** JVisu variable of type short */
    public  static final int T_SHORT = 5;
    /** JVisu variable of type integer */
    public  static final int T_INT = 6;
    /** JVisu variable of type float */
    public  static final int T_FLOAT = 7;
    /** JVisu variable of type stream */
    public  static final int T_STREAM = 8;
    /** JVisu variable of type bytearray */
    public  static final int T_BYTEARRAY = 9;
    /** JVisu variable of type bytearray */
    public  static final int T_SHORTARRAY = 10;
    /** JVisu variable of type bytearray */
    public  static final int T_INTARRAY = 11;
    /** JVisu variable of type bytearray */
    public  static final int T_FLOATARRAY = 12;
    /** JVisu variable of type bytearray */
    public  static final int T_DOUBLEARRAY = 13;


    
    
    /** Array of Classnames */
    private static String[] knownSubClasses = {
        null,
        "org.jvisu.comm.var.JVVarDouble",
        "org.jvisu.comm.var.JVVarString",
        "org.jvisu.comm.var.JVVarBool",
        "org.jvisu.comm.var.JVVarByte",
        "org.jvisu.comm.var.JVVarShort",
        "org.jvisu.comm.var.JVVarInt",
        "org.jvisu.comm.var.JVVarFloat",
        "org.jvisu.comm.var.JVVarStream",
        "org.jvisu.comm.var.JVVarByteArray",
        "org.jvisu.comm.var.JVVarShortArray",
        "org.jvisu.comm.var.JVVarIntArray",
        "org.jvisu.comm.var.JVVarFloatArray",
        "org.jvisu.comm.var.JVVarDoubleArray"
    };
    
    /** Variable configuration */
    protected JVVarCfg      varcfg;
    /** ArrayList to store the variable listeners */
    private ArrayList       varListeners;
    /** Index of variable in commserver */
    private int             srvIndex = -1;
    /** Elapsed time since last update (in simulation mode) */
    private int             elapsed;
    /** An user defined object, which can identify the variable */
    private Object          userObject;
    
    
    /** Create a new JVVar
     */
    public JVVar() {
        varListeners = new ArrayList();
    }
    
    /** Create a new JVVar
     *  @param cfg  variable configuration
     *  @param listener      listener for variable change events
     */
    public JVVar(JVVarCfg cfg, JVVarListener listener) {
        setVarCfg(cfg);
        varListeners = new ArrayList();
        addVarListener(listener);
    }
    
    /** Variable creator
     *  @param cfg a variable configuration, which should be created
     *  @param var a variable, which is of the same type and can be reused
     *  @param listener the variable listener, which wants to listen fot
     *         variable change events
     *  @return variable instance
     */
    public static JVVar CreateInstance(JVVarCfg cfg, JVVarListener listener) {
        JVVar var = null;
        if (cfg.getType() > 0 && cfg.getType() < knownSubClasses.length) {
            try {
                Class classDef = Class.forName(knownSubClasses[cfg.getType()]);
                Object obj = classDef.newInstance();
                var = (JVVar) obj;
            } catch (InstantiationException e) {
                System.out.println(e);
            } catch (IllegalAccessException e) {
                System.out.println(e);
            } catch (ClassNotFoundException e) {
                System.out.println(e);
            } catch (ClassCastException e) {
                System.out.println(e);
            } catch (Exception e) {
                System.out.println(e);
            }
            
        }
        if (var != null) {
            var.setVarCfg(cfg);
            var.addVarListener(listener);
        }
        return var;
    }
    
    /** Get the variable configuration
     *  @return configuration data
     */
    public JVVarCfg getVarCfg() {
        return new JVVarCfg(varcfg);
    }
    
    /** Get the variable configuration
     *  @param cfg configuration data
     */
    protected void setVarCfg(JVVarCfg cfg) {
        varcfg = new JVVarCfg(cfg);
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
     */
    public abstract boolean SetVarValue(ByteBuffer buf);
    
    /** Set the variable value from an object
     *  @param obj  object, which contains the new variable value
     *  @return     true, if successful
     */
    public abstract boolean SetVarValue(Object obj);
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public abstract boolean SetVarValue(String str);
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public abstract ByteBuffer GetVarValue(ByteBuffer buf);
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public abstract String toString();
    
    /** add the listener to the registered listeners
     *  @param listener the new listener, which has to be added
     */
    public void addVarListener(JVVarListener listener) {
        if (listener != null) {
            varListeners.add(listener);
            listener.VarChanged(this);
        }
    }
    
    /** remove the listener from the registered listeners
     *  @param listener the new listener, which has to be removed
     */
    public void delVarListener(JVVarListener listener) {
        varListeners.remove(listener);
    }
    
    /** Returns the variable symbol
     *  @return String which contains symbol or null
     */
    public String getSymbol() {
        return varcfg.getSymbol();
    }
    
    /** Get server index of variable
     * @return index in the server
     */
    public int getSrvIndex() {
        return srvIndex;
    }
    
    /** Set server index of variable
     * @return index in the server
     */
    public void setSrvIndex(int index) {
        this.srvIndex = index;
    }
   
    public void setSize(int size) { }
 
    /** Return, if this variable can be released.
     *  This can only be done, if the variable has no
     *  more registered listeners
     *  @return true, if the listeners array is empty
     */
    public boolean isRemovable() {
        return varListeners.isEmpty();
    }
    
    /** Notify all registered listeners, that the variable value
     *  has changed
     *  @param changed indicates, if the variable has changed
     */
    protected void NotifyListeners(boolean changed) {
        if (!changed && !varcfg.isForcedUpdated())
            return;
        for (int i = 0; i < varListeners.size(); i++)
            ((JVVarListener)varListeners.get(i)).VarChanged(this);
    }
    
    /** Get the time elapsed since last update
     *  @return Value of property elapsed.
     */
    public int getElapsed() {
        return this.elapsed;
    }
    
    /** Set the time elapsed since last update
     * @param elapsed New value of property elapsed.
     */
    public void setElapsed(int elapsed) {
        this.elapsed = elapsed;
    }

    /** Get the user object of this variable
     *  @return User object
     */
    public Object getUserObject() {
        return userObject;
    }
    
    /** Set the user object of this variable
     *  @param obj User object
     */
    public void setUserObject(Object obj) {
        userObject = obj;
    }
    
}
