package org.jvisu.panel.configurator;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.util.*;
import javax.swing.*;
import org.jvisu.base.*;
import org.jvisu.panel.*;
import org.jvisu.panel.configurator.events.*;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVPanelEditor.java</p>
 * $Id: JVPanelEditor.java 320 2007-09-11 11:57:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * JVPanelEditor is the visual editor component for JVPanel design.
 *  It consists of an JVPanelEdit and an JPanel, which is transparent and
 *  lies on top of JVPanelEdit.
 *  The top component receives all MouseEvents and is used to draw the
 *  selection rectangle.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVPanelEditor extends javax.swing.JScrollPane implements 
    PropertyChangeListener, MouseListener, MouseMotionListener, 
    ContainerListener, ComponentListener {
    
    private final static int CURSOR_TRESHOLD = 5;
    
    /** The editable subclass of JVPanel */
    private JVPanelEdit     pEdit;
    /** The panel, where pEdit is inside */
    private JPanel          pEPan;
    /** The top layer */
    private JVTopPanel      pTop;
    /* The Panel, which hosts pEdit and pTop */
    private JPanel          pPan;
    /** Current selection rectangle */
    private Rectangle       selection;
    /** Drag selection rectangle */
    private Rectangle       dragSelection;
    /** Position where mouse was pressed */
    private Point           dragStart;
    /** Component which is actually selected */
    private JComponent      selectedComp;
    /** Set, if the component was dragged */
    private boolean         dragged;
    /** Clickpoint */
    private Point           clickpoint;
    /** A element is selected and can be dropped into the design */
    private boolean         elementSelected;
    /** The popupmenu */
    private JPopupMenu      menu;
    /** Editor event support */
    private JVEditEventSupport  eventSupport = new JVEditEventSupport();
    
    /** Creates a new instance of JVSubPanel */
    public JVPanelEditor() {
        pPan= new JPanel();
        OverlayLayout overlay = new OverlayLayout(pPan);
        pPan.setLayout(overlay);
        
        pEdit = new JVPanelEdit();
        pEdit.addPropertyChangeListener(this);
        //pEdit.setEnabled(false);
        pEdit.addContainerListener(this);
        pEdit.addComponentListener(this);
        
        pEPan = new JPanel(new BorderLayout());
        pEPan.setBorder(new javax.swing.border.LineBorder(Color.WHITE, 10, true));
        pEPan.add(pEdit, BorderLayout.CENTER);
        
        pTop = new JVTopPanel();
        pTop.setOpaque(false);
        pTop.addMouseListener(this);
        pTop.addMouseMotionListener(this);
        pPan.setBackground(Color.LIGHT_GRAY);
        pPan.setBorder(new javax.swing.border.LineBorder(Color.LIGHT_GRAY, 10, true));
        
        pPan.add(pTop);
        pPan.add(pEPan);
        this.setViewportView(pPan);
        pEdit.setDesignsize(new Dimension(400,400));
         
        selection = new Rectangle();
        dragSelection = new Rectangle();
    }

    /** Add an listener 
     *  @param listener Listener to add to the listener list
     */
    public void addEditEventListener(JVEditEventListener listener) {
        eventSupport.addEditEventListener(listener);
    }

    /** Remove an listener 
     *  @param listener Listener to remove from the listener list
     */
    public void removeEditEventListener(JVEditEventListener listener) {
        eventSupport.removeEditEventListener(listener);
    }
    
    /** A method for the property cellpanels to fire a property
     *  change event.
     */
    /** Fire an editor event 
     *  @param evt the new editor event, which should be fired
     */
    public void fireEditEvent (JVEditEvent evt) {
        eventSupport.fireEditEvent(evt);
    }    
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        String key = evt.getPropertyName();
        if (evt.getSource() == pEdit) {
            // Make sure, that pTop has the same size as pEdit
            if (key.equals("preferredSize")) {
                Dimension size = new Dimension(pEdit.getPreferredSize());
                size.height += 20;
                size.width += 20;
                pEPan.setPreferredSize(size);
                pEPan.setMaximumSize(size);
                pTop.setPreferredSize(size);
                pTop.setMaximumSize(size);
                pPan.revalidate();
                pPan.repaint();
            }
        } else {
            /* A design element is selected. This Event is fired from
             * the component palette
             */
            if (key.equals("selectedElement"))
                elementSelected = (evt.getNewValue() != null);
        }
    }
    
    /** Container listener component added
     *  @param e the container event
     */
    public void componentAdded(ContainerEvent e) {
        if (e.getChild() instanceof JVBaseMembers) {
            JComponent comp = (JComponent) e.getChild();
            comp.addComponentListener(this);
            if (((JVBaseMembers)comp).isContainer()) {
                comp.addContainerListener(this);
            }
        }
        checkComponentEvent(e, JVEditEvent.COMP_ADDED);
    }
    
    /** Container listener component removed
     *  @param e the container event
     */
    public void componentRemoved(ContainerEvent e) {
        if (e.getChild() instanceof JVBaseMembers) {
            JComponent comp = (JComponent) e.getChild();
            comp.removeComponentListener(this);
            if (((JVBaseMembers)comp).isContainer()) {
                comp.removeContainerListener(this);
            }
        }
        checkComponentEvent(e, JVEditEvent.COMP_REMOVED);
    }

    /** Container listener fire editor event 
     *  @param evt the container event
     *  @param type the editor event type
     */
    private void checkComponentEvent(ContainerEvent evt, int type) {
        if (evt.getChild() instanceof JVBaseMembers)
            eventSupport.fireEditEvent(new JVEditEvent(this, type, evt.getChild()));
    }

    /** Not used here */
    public void componentHidden(ComponentEvent e) {
    }
    
    /** Component is moved. Redraw the selection
     *  rectangle
     */
    public void componentMoved(ComponentEvent e) {
        setComponentRectangle();
    }
    
    /** Component is moved. Redraw the selection
     *  rectangle
     */
    public void componentResized(ComponentEvent e) {
        setComponentRectangle();
    }
    
    /** Not used here */
    public void componentShown(ComponentEvent e) {
    }

    /** Get preview mode
     *  @return true, if preview mode is enabled
     */
    public boolean isPreviewMode () {
        return pEdit.isEnabled();
    }

    /** Set preview mode
     *  @param preview true, if preview mode is enabled
     */
    public void setPreviewMode (boolean on) {
        //pEdit.setEnabled(on);
        pTop.setVisible(!on);
    }
    
    /** Set the selected Component 
     * @param comp the selected component
     */
    public void setSelectedComponent(JComponent comp) {
        selectedComp = comp;
        setComponentRectangle();
    }
    
    /** Set the popupmenu
     *  @param men  the popup menu
     */
    public void setPopupMenu (JPopupMenu men) {
        menu = men;
    }
    
    /** Parse a given file
     *  @param fname filename to parse
     */
    public void parseFile(String fname) {
        pEdit.parseFile(fname);
    }
    
    /** Get editor panel */
    public JVPanelEdit getEditPanel() {
        return pEdit;
    }
    
    /** Fit the size
     */
    public void fitSize() {
        Dimension winsize = getSize();
        Dimension pansize = pTop.getSize();
        double fw = ((double) winsize.width) / ((double) pansize.width);
        double fh = ((double) winsize.height) / ((double) pansize.height);
        pEdit.setZoomfactor((fh < fw) ? fh : fw);
    }
    
    /** get the base address
     * @return base address
     */
    public String getBase() {
        return pEdit.getBase();
    }
    
    /** Set the base address.
     *  Inform all JVBeans components of the changes.
     *  @param newbase new base address.
     */
    public void setBase(String newbase) {
        pEdit.setBase(newbase);
    }
    
    /* Set the rectangle of the selected component */
    private void setComponentRectangle() {

        Rectangle r = pEdit.getCompRectangle(selectedComp, selection);
        if (!r.isEmpty()) {
            r.x += pEPan.getX() + pEdit.getX() - pTop.getX() - 2;
            r.y += pEPan.getY() + pEdit.getY() - pTop.getY() - 2;
            r.height += 4;
            r.width += 4;
        }
        pTop.setSelection(r);
    }

    /** Remove a component
     *  @param comp     the component, which will be removed
     */
    public void removeElement (JComponent comp) {
        if (comp == pEdit)
            return;
        JComponent parent = (JComponent) comp.getParent();
        parent.remove(comp);
        parent.revalidate();
        setSelectedComponent(parent);
        pEdit.repaint();
    }
    
    /** Read a configuration stream
     *  @param comp the component to which a element should be added
     *  @param in the inputstream to read
     */
    public void parseStream (JComponent comp, StringReader in) {
        pEdit.parseStream(comp, in);
    }
    
    /** Mouse clicked in top panel area.
     *  Select the underlying component.
     *  @param e mouse event
     */
    public void mouseClicked(MouseEvent e) {
        
        dragged = false;
        
        switch (e.getButton()) {
            case MouseEvent.BUTTON1: {
                Point p = e.getPoint();
                
                p.x -= pTop.getX() - pEPan.getX() + pEdit.getX();
                p.y -= pTop.getY() - pEPan.getY() + pEdit.getY();
                
                clickpoint = p;
                JComponent c = (JComponent) pEdit.findComponentAt(p);
                int evt;
                if (c != selectedComp) {
                    selectedComp = c;
                    setComponentRectangle();
                    evt = JVEditEvent.COMP_SELECTED;
                } else 
                    evt = JVEditEvent.COMP_CLICKED;
                eventSupport.fireEditEvent(new JVEditEvent(this, evt, selectedComp));
                break;
            }
            case MouseEvent.BUTTON3: {
                if (selectedComp == null || selection.isEmpty())
                    return;
                menu.show(pTop, e.getPoint().x, e.getPoint().y - 10);
            }
            default:
                return;
        }
    }
    
    /** Mouse dragged in top panel area.
     *  @param e mouse event
     *
     *  TODO: Use a custom cursor, when drag is outside the bounds !
     */
    public void mouseDragged(MouseEvent e) {
        
        if (selectedComp == null || selectedComp == pEdit ||
            selection.isEmpty() || e.getPoint() == null)
            return;
        
        dragged = true;
        
        int diffx = e.getPoint().x - dragStart.x;
        int diffy = e.getPoint().y - dragStart.y;
        
        dragSelection.setBounds(selection);        

        switch (pTop.getCursor().getType()) {
            case Cursor.MOVE_CURSOR:
                dragSelection.x += diffx;
                dragSelection.y += diffy;
                break;
                
            case Cursor.NW_RESIZE_CURSOR:
                dragSelection.x += diffx;
                dragSelection.width -= diffx;
            case Cursor.N_RESIZE_CURSOR:
                dragSelection.y += diffy;
                dragSelection.height -= diffy;
                break;
                
            case Cursor.SE_RESIZE_CURSOR:
                dragSelection.width += diffx;
            case Cursor.S_RESIZE_CURSOR:
                dragSelection.height += diffy;
                break;
                
            case Cursor.SW_RESIZE_CURSOR:
                dragSelection.height += diffy;
            case Cursor.W_RESIZE_CURSOR:
                dragSelection.x += diffx;
                dragSelection.width -= diffx;
                break;
                
            case Cursor.NE_RESIZE_CURSOR:
                dragSelection.y += diffy;
                dragSelection.height -= diffy;
                
            case Cursor.E_RESIZE_CURSOR:
                dragSelection.width += diffx;
                break;
        }
        pTop.setSelection(dragSelection);
        e.consume();
    }
    
    /** Mouse entered top panel area.
     *  @param e mouse event
     */
    public void mouseEntered(MouseEvent e) {
    }
    
    /** Mouse left top panel area.
     *  @param e mouse event
     */
    public void mouseExited(MouseEvent e) {
        dragged = false;
    }
    
    /** Mouse moved in top panel area.
     *  @param e mouse event
     */
    public void mouseMoved(MouseEvent e) {
        
        int cur = Cursor.DEFAULT_CURSOR;
        dragged = false;
        
        /* Anything selected ? */
        if (!selection.isEmpty() && selection.contains(e.getPoint()) && 
                selectedComp != null && selectedComp != pEdit && !elementSelected) {
            if (Math.abs(selection.x - e.getPoint().x) < CURSOR_TRESHOLD) {
                cur = Cursor.W_RESIZE_CURSOR;
                if (Math.abs(selection.y - e.getPoint().y) < CURSOR_TRESHOLD)
                    cur = Cursor.NW_RESIZE_CURSOR;
                if (Math.abs((selection.y + selection.height) - e.getPoint().y) < CURSOR_TRESHOLD)
                    cur = Cursor.SW_RESIZE_CURSOR;
            } else if (Math.abs((selection.x + selection.width) - e.getPoint().x) < CURSOR_TRESHOLD) {
                cur = Cursor.E_RESIZE_CURSOR;
                if (Math.abs(selection.y - e.getPoint().y) < CURSOR_TRESHOLD)
                    cur = Cursor.NE_RESIZE_CURSOR;
                if (Math.abs((selection.y + selection.height) - e.getPoint().y) < CURSOR_TRESHOLD)
                    cur = Cursor.SE_RESIZE_CURSOR;
            } else if (Math.abs(selection.y - e.getPoint().y) < CURSOR_TRESHOLD) {
                cur = Cursor.N_RESIZE_CURSOR;
            } else if (Math.abs((selection.y + selection.height) - e.getPoint().y) < CURSOR_TRESHOLD) {
                cur = Cursor.S_RESIZE_CURSOR;
            } else {
                if (selectedComp.getParent().getLayout() instanceof SpringLayout)
                    cur = Cursor.MOVE_CURSOR;
                else
                    cur = Cursor.DEFAULT_CURSOR;
            }
        }
        pTop.setCursor(Cursor.getPredefinedCursor(cur));
        e.consume();
    }
    
    /** Mouse pressed in top panel area.
     *  @param e mouse event
     */
    public void mousePressed(MouseEvent e) {

        dragged = false;

        switch (e.getButton()) {
            case MouseEvent.BUTTON1: {
                if (pTop.getCursor().getType() != Cursor.DEFAULT_CURSOR) {
                    dragStart = e.getPoint();
                    dragSelection.setBounds(selection);
                }
                break;
            } // End BUTTON1
        }
        e.consume();
    }
    
    /** Mouse released in top panel area.
     *  @param e mouse event
     */
    public void mouseReleased(MouseEvent e) {
        
        switch (e.getButton()) {
            
            case MouseEvent.BUTTON1: {
                if (dragged) {
                    // Move and resize the component
                    Rectangle rect = new Rectangle(dragSelection);
                    rect.x -= selection.x;
                    rect.y -= selection.y;
                    rect.height -= selection.height;
                    rect.width -= selection.width;
                    
                    JVBaseMembers bm = (JVBaseMembers) selectedComp;
                    Point p = new Point(bm.getDesignpos());
                    p.setLocation(p.getX() + rect.getX() / pEdit.getZoomfactor(), 
                                p.getY() + rect.getY() / pEdit.getZoomfactor());
                    bm.setDesignpos(p);
                    
                    Dimension size = new Dimension(selectedComp.getPreferredSize());
                    size.width += rect.width;
                    size.height += rect.height;
                    size.setSize(size.getWidth() / pEdit.getZoomfactor(), 
                        size.getHeight() / pEdit.getZoomfactor());
                    ((JVBaseMembers)selectedComp).setDesignsize(size);
                    selection.setBounds(dragSelection);
                    pTop.setSelection(selection);
                }
                repaint();
                break;
            } // End BUTTON1
            default:
                break;
        }
        dragged = false;
        pTop.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        e.consume();
    }
}
