package org.jvisu.util.param;

import java.awt.*;
import org.jvisu.comm.var.JVVar;

/** Parameter parse functions.
 *
 *  <p>$Id: JVParams.java 111 2003-11-10 10:25:54Z tgl $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p> This class contains only static functions for retrieving
 *  parameters from a string. The functions are called with a string
 *  and a default value, which is returned if the string is null or
 *  does not contain a valid representation of the given data type.
 *  </p>
 *  <p>Implement all known datatypes</p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 * @since JVisu 1.0
 */

public class JVParams {

    /** ParamType tries to figure out, which type of param is given
     * @param param String, which has to be analysed
     * @return one of the type constants
     */
    public static int ParamType (String param) {
        if (param == null)
            return JVVar.T_STRING;
        if (param.compareTo("true") == 0)
            return JVVar.T_BOOL;
        if (param.compareTo("false") == 0)
            return JVVar.T_BOOL;
        try { 
            Double.parseDouble(param); 
            return JVVar.T_DOUBLE;
        } catch (NumberFormatException e) { }
        return JVVar.T_STRING;
    }
    
    /** StringToBool tries to parse an boolean Value from a given String
     *  If the parsing failes the given default value is returned
     *  @param newval String, which has to be parsed
     *  @param defval Value, which is used as default, if newval cannot be identified
     *  @return the new or the default value
     */
    public static boolean StringToBool (String newval, boolean defval) {
        if (newval == null)
            return defval;
        if (newval.compareTo("true") == 0 || newval.compareTo("1") == 0)
            return true;
        if (newval.compareTo("false") == 0  || newval.compareTo("0") == 0)
            return false;
        return defval;
    }

    /** StringToInt tries to parse an integer Value from a given String
     *  If the parsing failes the given default value is returned
     *  @param newval String, which has to be parsed
     *  @param defval Value, which is used as default, if newval cannot be identified
     *  @return the new or the default value
     */
    public static int StringToInt (String newval, int defval) {
        if (newval == null || newval.length() == 0)
            return defval;
        try { return Integer.parseInt(newval, 10); } 
        catch (Exception e) { return defval; }
    }

    /** StringToInt tries to parse an double Value from a given String
     *  If the parsing failes the given default value is returned
     *  @param newval String, which has to be parsed
     *  @param defval Value, which is used as default, if newval cannot be identified
     *  @return the new or the default value
     */
    public static double StringToDouble (String newval, double defval) {
        if (newval == null || newval.length() == 0)
            return defval;
        try { return Double.parseDouble(newval); } 
        catch (Exception e) { return defval; }
    }

    /** StringToDimension tries to parse a given String as a Dimension.
     *  If this failes, the given default is returned
     *  @param val String, which describes the dimension
     *  @param defval default value, if parsing failes
     *  @return the parsed or the default dimension
     */
    public static Dimension StringToDimension (String val, Dimension defval) {
        int sep = val.indexOf(',');
        try {
            int w = Integer.parseInt(val.substring(0,sep).trim());
            int h = Integer.parseInt(val.substring(sep+1).trim());
            return new Dimension(w, h);
        } catch (Exception e) {
            return defval;
        }
    }

    /** StringToPoint tries to parse a given String as a Point.
     *  If this failes, the given default is returned
     *  @param val String, which describes the point
     *  @param defval default value, if parsing failes
     *  @return the parsed or the default point
     */
    public static Point StringToPoint (String val, Point defval) {
        int sep = val.indexOf(',');
        try {
            int x = Integer.parseInt(val.substring(0,sep).trim());
            int y = Integer.parseInt(val.substring(sep+1).trim());
            return new Point(x, y);
        } catch (Exception e) {
            return defval;
        }
    }

    /** StringToFont tries to parse a given String as a Font.
     *  If this failes, the given default is returned
     *  @param val String, which describes the font
     *  @param defval default value, if parsing failes
     *  @return the parsed or the default font
     */
    public static Font StringToFont (String val,Font defval) {
        try { 
            int sep1 = val.indexOf(',');
            int sep2 = val.indexOf(',', sep1 + 1);
            int style = Integer.parseInt(val.substring(sep1 + 1, sep2).trim());
            int size = Integer.parseInt(val.substring(sep2 + 1).trim());
            return new Font(val.substring(0,sep1).trim(),style, size);
        } catch (Exception e) {
            return defval;
        }
    }
    
    /** StringToColor tries to parse a given String as a Color.
     *  If this failes, the given default color is returned
     *  @param newcolor String, which describes the color
     *  @param defcolor Color, which is used as default, if newcolor cannot be parsed
     *  @return the new or the default Color
     */
    public static Color StringToColor (String newcolor, Color defcolor) {
        try { 
            int sep1 = newcolor.indexOf(',');
            int sep2 = newcolor.indexOf(',', sep1 + 1);
            int r = Integer.parseInt(newcolor.substring(0,sep1).trim());
            int g = Integer.parseInt(newcolor.substring(sep1 + 1, sep2).trim());
            int b = Integer.parseInt(newcolor.substring(sep2 + 1).trim());
            return new Color(r, g, b);
        } catch (Exception e) {
            return defcolor;
        }
    }
    
    /** Return a String representation of known objects
     *  @param obj object to convert
     *  @return String representation or null
     */
    public static String ObjectToString (Object obj) {
        
        if (obj == null)
            return "";
        //Class clazz = obj.getClass();
        
        if ( (obj instanceof Integer) ||
             (obj instanceof Long) ||
             (obj instanceof Short) ||
             (obj instanceof Byte) ||
             (obj instanceof Boolean) ||
             (obj instanceof Float) ||
             (obj instanceof Double) ||
             (obj instanceof String))
            return obj.toString();

        if (obj instanceof Color) {
            Color col = (Color) obj;
            return (col.getRed() + "," + col.getGreen() + "," + col.getBlue());
        }
        
        if (obj instanceof Font) {
            Font font = (Font) obj;
            return (font.getName() + "," + font.getStyle() + "," + font.getSize());
        }
        
        if (obj instanceof Point) {
            Point p = (Point) obj;
            return (p.x + "," + p.y);
        }
        
        if (obj instanceof Dimension) {
            Dimension d = (Dimension) obj;
            return (d.width + "," + d.height);
        }
        return null;
    }
}
